
#ifndef ANGLEPOTENTIAL_H
#define ANGLEPOTENTIAL_H

#include "rumd/Potential.h"


///////////////////////////////////////////////////////
// Abstract BondPotential class
///////////////////////////////////////////////////////

class AnglePotential : public Potential{
 public:
  AnglePotential() : num_threads(32) {}
  bool EnergyIncludedInParticleSum() const { return true; };
  void Initialize() {}
protected:
  unsigned num_threads;
};


// Cosine squared potential
class AngleCosSq : public AnglePotential{
 private:
  AngleCosSq(const AngleCosSq&);
  AngleCosSq& operator=(const AngleCosSq&);
  
 public:
  AngleCosSq();
  void CalcF(bool initialize, bool calc_stresses);
  double GetPotentialEnergy();
};

// Angle squared potential
class AngleSq : public AnglePotential{
 private:
  AngleSq(const AngleSq&);
  AngleSq& operator=(const AngleSq&);

 public:
  AngleSq();
  void CalcF(bool initialize, bool calc_stresses);
  double GetPotentialEnergy();
};



///////////////////////////////////////////////////////
// Kernel prototypes
///////////////////////////////////////////////////////

template <int energy, class Simbox> 
__global__ void CosSq( float4 *r, float4 *f,
		       uint4 *alist, float2 *parameter,
		       float *d_epot_angle, float *d_angles,
		       unsigned num_angles, Simbox *simbox, float *simboxpointer );

template <int energy, class Simbox>
__global__ void SquaredAngle( float4 *r, float4 *f,
                       uint4 *alist, float2 *parameter,
                       float *d_epot_angle, float *d_angles,
                       unsigned num_angles, Simbox *simbox, float *simboxpointer );


//////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////
__global__ void AngleResetForces( float4 *f, float4 *w );
__device__ inline void AngleAtomicFloatAdd(float *address, float val);

#endif
