
#ifndef DIHEDRALPOTENTIAL_H
#define DIHEDRALPOTENTIAL_H

#include "rumd/Potential.h"

#define DIHEDRAL_EPS 1.0e-3


///////////////////////////////////////////////////////
// Abstract Potential class
///////////////////////////////////////////////////////

class DihedralPotential : public Potential{
 public:
  DihedralPotential() : num_threads(16) {}
  bool EnergyIncludedInParticleSum() const { return true; };
  void Initialize() {}
protected:
  unsigned num_threads;
};


// Ryckaert-Bellemans potential
class DihedralRyckaert : public DihedralPotential{
 private:
  DihedralRyckaert(const DihedralRyckaert&);
  DihedralRyckaert& operator=(const DihedralRyckaert&);
  
 public:
  DihedralRyckaert();
  void CalcF(bool initialize, bool calc_stresses);
  double GetPotentialEnergy();
};

// Periodic dihedral potential
class PeriodicDihedral : public DihedralPotential{
 private:
  PeriodicDihedral(const PeriodicDihedral&);
  PeriodicDihedral& operator=(const PeriodicDihedral&);

 public:
  PeriodicDihedral();
  void CalcF(bool initialize, bool calc_stresses);
  double GetPotentialEnergy();
};


///////////////////////////////////////////////////////
// Kernel prototypes
///////////////////////////////////////////////////////

template <int energy, class Simbox> 
__global__ void Ryckaert( float4 *r, float4 *f, 
			  uint4 *dlist, uint1 *dtype, float *plist, 
			  float *d_epot_dihedral, float *d_dihedrals,
			  unsigned num_dihedrals, Simbox *simbox, float *simboxpointer  );

template <int energy, class Simbox>
__global__ void PeriodicDih( float4 *r, float4 *f,
                          uint4 *dlist, uint1 *dtype, float *plist,
                          float *d_epot_dihedral, float *d_dihedrals,
                          unsigned num_dihedrals, Simbox *simbox, float *simboxpointer  );

/////////////////////////////////////////////////////////

__global__ void DihedralResetForces( float4 *f, float4 *w );
__device__ inline void DihedralAtomicFloatAdd(float *address, float val);


#endif
